#! /usr/bin/env python

# numSecStrHavingKbasepairs.py
# P.Clote

# Computes the number of sec str having k basepairs, assuming
# every base can base pair with every other base. 

import sys,math
THRESHOLD = 3
DEBUG     = 0

def maxNumBasePairs(n):
  return (n-THRESHOLD)/2


def locOpt(n):  
  #D[(m,k)] is number of sec str on {1,...,m} in homopolymer with k base pairs
  D = {}
  for m in range(THRESHOLD+2):     #first handle 0<=m<=THRESHOLD+1
    D[(m,0)] = 1L #count empty structure with no base pairs
    for k in range(1,maxNumBasePairs(m)+1): 
      D[(m,k)] = 0L
      # |j-i|< THRESHOLD, so can't base pair
  for m in range(THRESHOLD+2,n+1): #now handle THRESHOLD+2<=m<=n/2
    D[(m,0)] = 1L  #count empty structure with no base pairs
    for k in range(1,maxNumBasePairs(m)+1): 
      if DEBUG: print "k:%d" %k
      sum = 0L
      #Case 1: m does not base pair
      if k <= maxNumBasePairs(m-1):
        sum += D[(m-1,k)]
      #Case 2: (1,m) is base pair
      if k-1 <= maxNumBasePairs(m-2):
        sum += D[(m-2,k-1)]  
      #Case 3: (i,m) base pair for some 1<=i<=m-THRESHOLD-1
      for i in range(1,m-THRESHOLD):  
        if DEBUG: print "i:%d" %i
        for k0 in range(k):  
          #distribute k0 base pairs in left portion, for 0<=k0<k
          #k0 base pairs in left portion, 1 fewer than k since (i,m) is basepair
          k1 = k-k0-1   # so k0+k1+1 = k
          leftSide = i-1; rightSide = m-1-(i+1)+1
          if k0<=maxNumBasePairs(leftSide) and k1<=maxNumBasePairs(rightSide):
            sum += D[(leftSide,k0)]*D[(rightSide,k1)]
            if DEBUG: 
              print leftSide,rightSide, D[(leftSide,k0)],D[(rightSide,k1)]
      D[(m,k)] = sum
  return D


def main(n):
  D = locOpt(n)
  print "Length of RNA: %d" % n
  print "Num basepairs\tNum sec str with this number of base pairs"
  for k in range(maxNumBasePairs(n)+1):
    print "%d\t%s" % (k,D[(n,k)])
  

if __name__ == '__main__':
  if len(sys.argv)!= 2:
    print "Usage: %s integer" % sys.argv[0]
    sys.exit(1)
  main(int(sys.argv[1]))


